var PhenoTips = (function (PhenoTips) {
  var generateTabs = function (onSimilarCasesPage) {

    var _generateSimilarCasesTab = function (isActive) {
      var url = XWiki.currentDocument.getURL('view', 'sheet=PhenoTips.SimilarCases');
      var tab = _generateTab(
        'similar-cases-tab', // element id
        isActive,
        "Matching patients", //displayed text
        url // url for inactive tabs
      );

      if (!isActive) {
        tab.observe('click', function(event) {
          event.stop();
          PhenoTips.widgets.FormUtils.getFormState().saveIfFormDirty( function () {
            window.location = url;
          });
        });
      }

      return tab;
    };

    var _generateInactivePatientsTab = function () {
      return _generateTab(
        'document-title',
        false,
        XWiki.currentDocument.page,
        XWiki.currentDocument.getURL(XWiki.contextaction)
      );
    };

    var _generateTab = function (id, isActive, text, url) {
      var elt = new Element('div', {'class' : 'title-tab' + (isActive? ' active' : ''), id : id});
      if (isActive) {
        var title = new Element('h1').update(text)
        elt.update(title);
      } else {
        var link = new Element('a', {'href' : url});
        link.update(text);
        elt.update(link);
      }
      return elt;
    };

    var _hideElement = function (id) {
      $(id) && $(id).hide();
    };

    if (onSimilarCasesPage) {
      $('body').addClassName('matching-patients');
      // Change patient's page tab to be inactive
      $('document-head') && $('document-title').insert({after: _generateInactivePatientsTab()});
      $('document-title') && $('document-title').remove();
      ['document-authors', 'patient-record-actions', $$('.bottombuttons')[0]].each(_hideElement);
    }

    // Add a new tab in the title bar
    $('document-head') && $('document-title').insert({after: _generateSimilarCasesTab(onSimilarCasesPage)});
  };

  var _generateFailureReason = function(status) {
      switch(status) {
        case 401:
          return "PhenomeCentral is not authorized to contact selected MME server";
        case 400:
          return "The request has invalid syntax";
        case 204:
          return "phenotips.similarCases.noMatchRequest";
        case 409:
          return "Can not send an MME request for this patient";
        case 503:
          return "Unable to connect to remote MME server";
        case 403:
          return "User does not have enough rights to send match requests for this patient";
        case 406:
          return "Remote MME server rejected match request";
        case 415:
          return "Remote MME server is using unsupported MME protocol version";
        case 500:
          return "An unexpected error occurred";
        default:
          return "An unexpected error occurred";
      }
  };

  var initRefreshRemoteMatchesSelect = function() {
    if (!$('similarity-results-refresh')) {
      return;
    }

    this._outstandingRefreshCalls = 0;     // the number of "refresh matches" AJAX calls to various servers still in progress
    this._outstandingSuccessfulCalls = 0;  // the number of successfully completed refresh calls for which results are not displayed yet

    var _refreshMatches = function(url, serverId, notification) {
      var _this = this;

      var _getServerName = function(serverId) {
        var name = $('remote-select-dropdown').down('a.' + serverId);
        if (name) {
          return name.firstChild.textContent;
        } else {
          return serverId;
        }
      };

      var serverName = _getServerName(serverId);

      new Ajax.Request(url + "?method=PUT", {
          contentType : 'application/json',
          parameters : {'serverId' : serverId},
          onCreate : function () {
            _this._outstandingRefreshCalls += 1;
            $("panels-livetable-ajax-loader").show();
            // disable the refresh matches button, to avoid accidental double-refreshes (especially to MME servers)
            $('refresh-all-button').disable();
            $('refresh-remote-select').disable();
            // grey out the main matches table as any user interaction may be disrupted by the refreshed matches after this call is complete
            $('table-container').addClassName("greyed-out");
          },
          onSuccess : function(response) {
            var message = (serverId == "local") ? "Successfully refreshed local matches" : "Successfully refreshed matches for server " + serverName;
            notification.replace(new XWiki.widgets.Notification(message, "done"));
            _this._outstandingSuccessfulCalls += 1;
          },
          onFailure : function(response) {
            var error = _generateFailureReason(response.status);
            var message = "Error sending request for server " + serverName + ": " + error;
            notification.replace(new XWiki.widgets.Notification(message, "error"));
          },
          onComplete: function() {
            _this._outstandingRefreshCalls -= 1;

            if (_this._outstandingRefreshCalls == 0) {
                // the last outstanding request completed - restore all UI elements
                $("panels-livetable-ajax-loader").hide();
                $('refresh-all-button').enable();
                $('refresh-remote-select').enable();
                $('table-container').removeClassName("greyed-out");

                if (_this._outstandingSuccessfulCalls > 0) {
                  // at least one of the refresh calls produced new matches - need to request a match reload
                  var event = { 'updatedMatches' : 'success' };
                  document.fire("matches:refreshed", event);
                  _this._outstandingSuccessfulCalls = 0;
                }
            }

            // success or fail, refresh server status in the dropdown after every call
            fetchLastUpdatedTime();
          }
      });
    };

    var urlRefreshMatches = XWiki.contextPath + '/rest/matches/patients/' + XWiki.currentDocument.page;
    var refreshingNotification = new XWiki.widgets.Notification("Refreshing", "inprogress", {'inactive': true});

    // Refresh local matches
    $('refresh-all-button') && $('refresh-all-button').observe("click", function(event) {
        event.stop();
        refreshingNotification.show();
        var allServers = $$('#remote-select-dropdown a');
        allServers.each(function (server) {
          (server.className != "all-remotes") && _refreshMatches(urlRefreshMatches, server.className, refreshingNotification);
        }.bind(this));
    });

    $('refresh-remote-select').observe("click", function(event) {
        event.stop();
        $('remote-select-dropdown').toggleClassName("show");
        $('body').addEventListener('click', dropdownCloser, false);
    });

    $$(".dropdown-menu a").each(function (el) {
        el.observe("click", function(event) {
            $('remote-select-dropdown').removeClassName("show");
            $('body').removeEventListener('click', dropdownCloser, false);

            refreshingNotification.show();
            _refreshMatches(urlRefreshMatches, this.className, refreshingNotification);
        });
    }.bind(this));

    var dropdownCloser = function(event) {
        // close remote servers dropdown menu on click outside
        var el = event.element();
        if (!el.up('#remote-select-dropdown')) {
            $('remote-select-dropdown').removeClassName("show");
            $('body').removeEventListener('click', dropdownCloser, false);
        }
    };

    var fetchLastUpdatedTime = function() {
        // fetch last update day and append to the server dropdown list for matches refresh
        new Ajax.Request(XWiki.contextPath + "/rest/matches/patients/" + XWiki.currentDocument.page + "/updated?method=GET", {
            contentType : 'application/json',
            onSuccess : function(response) {
                if (response && response.responseJSON) {
                    var matchUpdateStatus = response.responseJSON;

                    // exclude local server, as we always show "matches were just computed" for local DB
                    delete matchUpdateStatus.local;

                    Object.keys(matchUpdateStatus).each(function (serverId) {

                        var serverDropdownEl = $$(".dropdown-menu a." + serverId)[0];
                        if (!serverDropdownEl) {
                            return;
                        }

                        var serverStatus = matchUpdateStatus[serverId];

                        var lastRunWasAnError = serverStatus.hasOwnProperty("lastMatchUpdateDate")
                                                && serverStatus.lastMatchUpdateDate != null
                                                && new Date(serverStatus.lastMatchUpdateDate)
                                                && serverStatus.hasOwnProperty("lastMatchUpdateError")
                                                && serverStatus.hasOwnProperty("lastMatchUpdateErrorCode");

                        var lastGoodRunDate = (serverStatus.hasOwnProperty("lastSuccessfulMatchUpdateDate")
                                               && serverStatus.lastSuccessfulMatchUpdateDate != null
                                               && new Date(serverStatus.lastSuccessfulMatchUpdateDate))
                                              ? new Date(serverStatus.lastSuccessfulMatchUpdateDate)
                                              : undefined;

                        var mmeTimeMessage = '';
                        if (lastGoodRunDate) {
                            var now = new Date();

                            // if matches were updated less than 10 minutes ago
                            if (now.getTime() - lastGoodRunDate.getTime() < 1000*60*10) {
                                mmeTimeMessage = "Matches computed just now";
                            } else {
                                mmeTimeMessage = "Matches computed on _TIME_"
                                                 .replace("_TIME_", lastGoodRunDate.toISOString().split('T')[0]);
                            }
                        } else {
                            if (lastRunWasAnError) {
                                mmeTimeMessage = "Matches never successfully computed";
                            } else {
                                mmeTimeMessage = "Matches never computed";
                            }
                        }

                        if (!serverDropdownEl.down('.match-timeline-info')){
                            serverDropdownEl.insert(new Element('div', {'class' : 'match-timeline-info'}));
                        }

                        serverDropdownEl.down('.match-timeline-info').update(mmeTimeMessage);

                        if (lastRunWasAnError) {
                            var errorDate = new Date(serverStatus.lastMatchUpdateDate).toISOString().split('T')[0];
                            var errorMessage = "Latest match computation on _TIME_ generated an error:"
                                                 .replace("_TIME_", errorDate);
                            var errorDiv = new Element('div', {'class' : 'match-update-error-info'}).update(errorMessage);
                            var errorDetailsDiv = new Element('div', {'class' : 'match-update-error-details'}).update(_generateFailureReason(serverStatus.lastMatchUpdateErrorCode));

                            serverDropdownEl.down('.match-timeline-info').insert(errorDiv).insert(errorDetailsDiv);
                        }
                    });
                }
            }
        });
    };

    fetchLastUpdatedTime();
  };

  var fetchConsents = function(callback) {
    // Get local consents via REST and check if "real" consent is granted
    new Ajax.Request(XWiki.contextPath + "/rest/patients/" + XWiki.currentDocument.page + "/consents?method=get", {
      onSuccess : function(response) {
        var consents = response.responseJSON;
        if (!consents || consents.length == 0) {
          return;
        }
        // if "real" consent not granted - generate tab
        var realGranted = consents.filter( function(consent) { return consent.id == "real" && consent.status == "yes"; } );
        if (realGranted.length > 0) {
          callback(false);
        }
      }
    });
  };

  var init = function() {
    // Quick hack to confirm we're on a patient page and widgets.Matcher is not initialized yet:
    if (!$('patient-record-actions') || $('similar-cases-tab')) {
      return;
    }

    var onSimilarCasesPage = $('similar-cases-container');

    if (onSimilarCasesPage) {
      generateTabs(onSimilarCasesPage);
      initRefreshRemoteMatchesSelect();
    } else {
      // Matched patient's tab is not active
      // generate the tab for similar cases only if the "real" consent is granted

      // if consents are already available from the patient page
      if ($('patient-consents-json')) {
        // Check all required "real" consent:
        var consents = [];
        try {
          consents = JSON.parse($('patient-consents-json').value);
        } catch(err) {
          // console && console.log(err);
        }

        if (!consents || consents.length == 0) {
          return;
        }

        var realGranted = consents.filter( function(consent) { return consent.id == "real" && consent.status == "yes"; } );
        if (realGranted.length > 0) {
          generateTabs(onSimilarCasesPage);
        }
      } else {
        fetchConsents(generateTabs);
      }
    }
  };

  (XWiki && XWiki.domIsLoaded && init()) || document.observe("xwiki:dom:loaded", init);

  // End PhenoTips augmentation.
  return PhenoTips;
}(PhenoTips || {}));
