var PhenomeCentral = (function (PhenomeCentral) {
  var widgets = PhenomeCentral.widgets = PhenomeCentral.widgets || {};
  widgets.JournalSubmission = Class.create({
    JOURNALS : [
          ],

    initialize : function (permissionsManager) {
      if (!permissionsManager || permissionsManager.entityType != 'patients' || this.JOURNALS.length == 0) {
        return
      }
      this._permissionsManager = permissionsManager;
      this._journalsManager = permissionsManager._generateSection(
        'manage-journals',
        'Journals',
        'If this case is being submitted to one of the following journals, please check the appropriate box. You give the journal permission to access this record, and to make this record public if the case is published.'
      );
      permissionsManager._collaboratorsManager.insert({
        after: this._journalsManager
      });
      this._initJournalsManager();
      this._permissionsManager._container.observe('phenotips:permissions-manager:updated', this._updateJournals.bind(this));
    },

    _initJournalsManager : function() {
      var _this = this;
      var table = new Element('table', {id : 'journals'});
      this._journalsManager.insert(table);

      this.JOURNALS.each(function (j) {
        var input = new Element ('input', {'type' : 'checkbox', 'name' : 'journal', 'value' : j.id});
        var label = new Element('label').insert(input).insert(j.name);
        var row = new Element('tr')
               .insert(new Element('td').insert(label));
        table.insert(row);

        input.observe('change', function(event) {
          if (event.target) {
            if (event.target.checked === false) {
              // Remove journal collaborator
              console && console.log('Removing journal: ' + j.id);
              _this._removeJournal(j.id);
            } else {
              console && console.log('Adding journal: ' + j.id);
              _this._addJournal(j);
            }
          }
        });
      });
    },

    _addJournal : function (j, highlight) {
      var row = this._permissionsManager._generateCollaborator(j, highlight);
      row.addClassName('journal');
      // Ensure checkbox is checked
      var checkbox = this._journalsManager.down('input[type="checkbox"][value="' + j.id + '"]');
      if (checkbox) {
        console && console.log('Found checkbox... checking');
        checkbox.checked = true;
     }
     this._permissionsManager._collaboratorsList.insert(row);
   },

    _removeJournal : function (id) {
      // Remove hidden journal collaborator
      this._permissionsManager._removeCollaborator(id);
    },

    _updateJournals : function () {
      // Clear journal selection
      var checkboxes = this._journalsManager.select('input[type="checkbox"][name="journal"]');
      for (var i = 0; i < checkboxes.length; i++) {
        checkboxes[i].checked = false;
      }
      // Collaborator journals are already added, just add the correct class name and check the checkbox
      var _this = this;
      this._permissionsManager._collaboratorsList.select('tr input[type="hidden"][name="collaborator"').each(function(c) {
         _this._journalsManager.select('input[type="checkbox"][name="journal"][value$="' + c.value + '"]').each(function (cb) {
            console && console.log('Found checkbox for ' + c.value + '... checking');
            cb.checked = true;
            var cRow = c.up('tr');
            cRow.addClassName('journal');
         });
      });
    }
  });
  return PhenomeCentral;
}(PhenomeCentral || {}));

document.observe("phenotips:permissions-manager:loaded", function(event) {
  new PhenomeCentral.widgets.JournalSubmission(event && event.memo && event.memo.permissionsManager);
});
